;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# dataflash file manager 							#
;# copyright (c) 2006/2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
fman_main:	sts	libmio_channel,byte0	;output to screen
		sts	libmio_vidmode,byte0	;standard video mode
		ldi	XL,0x0a			;color scheme (cyan on black)
		sts	libmio_color,XL
		ldi	YL,0			;maximal sized box
		ldi	YH,0
		ldi	ZL,29
		ldi	ZH,22
		libmio_whbox
		ldi	XL,0xa0			;color scheme (cyan on black)
		sts	libmio_color,XL
		libmio_thistext
		.db 1,2,"DFLASH size: ",0
		call	fsys_maxpage		;get flash status
		cpi	tempreg1,0x0f		;64K
		brne	fman_main_001
		libmio_thistext
		.db 255,"64 KiByte",0,0
		rjmp	fman_main_02
		
fman_main_001:	cpi	tempreg1,0x7f		;0,5M
		brne	fman_main_002
		libmio_thistext
		.db 255,"512 KiByte",0
		rjmp	fman_main_02

fman_main_002:	cpi	tempreg1,0xff		;1M
		brne	fman_main_01
		libmio_thistext
		.db 255,"1 MiByte",0
		rjmp	fman_main_02

fman_main_01:	libmio_thistext
		.db 255,"0 Byte",0

fman_main_02:	call	fsys_checkf		;test for valid format
		cpi	tempreg1,0x01		;OK
		breq	fman_main_03
		rcall	fman_nfmenu
fman_mainf_1:	libmio_waitkey			;wait for key
		cpi	tempreg1,0xed		;ESC
		brne	fman_mainf_2
		rjmp	fman_wnk		;wait for no key and exit
fman_mainf_2:	cpi	tempreg1,0xf4		;F4=rescan
		brne	fman_mainf_3
		rjmp	fman_main
fman_mainf_3:	cpi	tempreg1,0xf1		;F1=format
		brne	fman_mainf_1
		rcall	fman_fmt		;format
		rjmp	fman_main

fman_main_x2:	rcall	fman_nomenu
fman_main_x3:	libmio_waitkey			;wait for key
		cpi	tempreg1,0xed		;ESC
		brne	fman_main_x4
		rjmp	fman_wnk		;wait for no key and exit
fman_main_x4:	cpi	tempreg1,0xf4		;rescan
		brne	fman_main_x3
		rjmp	fman_main


;------------------------------------------------------------------------------
;the main loop
;------------------------------------------------------------------------------
fman_main_03:	ldi	r19,0xdd		;dummy value
		sts	libmio_lastkey,byte0
	
fman_main_09:	call	libmio_scrs		;enable screenshots
		lds	tempreg1,libmio_kbdstate
		andi	tempreg1,0x04		;mask control bit
		cp	tempreg1,r19
		breq	fman_main_09c		;equal
		mov	r19,tempreg1		;
		cpi	tempreg1,0x00		;normal
		brne	fman_main_09a		;not
		rcall	fman_smenu		;standard menu
		rjmp	fman_main_09

fman_main_09a:	rcall	fman_cmenu		;control menu
		rjmp	fman_main_09

fman_main_09c:	lds	tempreg1,libmio_lastkey	;get key
		cpi	tempreg1,0x00
		breq	fman_main_09		
		

;------------------------------------------------------------------------------
;esc key
;------------------------------------------------------------------------------
fman_main_30:	cpi	tempreg1,0xed		;esc
		brne	fman_main_40
		rjmp	fman_wnk		;wait for no key and exit
		

;------------------------------------------------------------------------------
;check control key
;------------------------------------------------------------------------------
fman_main_40:	lds	tempreg2,libmio_kbdstate
		andi	tempreg2,0x04		;mask control bit
		breq	fman_main_50		;no ctrl
		
;------------------------------------------------------------------------------
;CTRL+F1 key (delete)
;------------------------------------------------------------------------------
fman_main_c0:	cpi	tempreg1,0xf1		;F1
		brne	fman_main_d0
		call	fsys_delete		
		rjmp	fman_main

;------------------------------------------------------------------------------
;CTRL+F4 key (format)
;------------------------------------------------------------------------------
fman_main_d0:	cpi	tempreg1,0xf4		;F4
		brne	fman_main_e0
		rcall	fman_fmt		;format
		rjmp	fman_main		;restart		
fman_main_e0:	rjmp	fman_main_09		;no active key

		
;------------------------------------------------------------------------------
;F1 key (info)
;------------------------------------------------------------------------------
fman_main_50:	cpi	tempreg1,0xf1		;F1
		brne	fman_main_60
		rcall	fman_info		
		rjmp	fman_main

;------------------------------------------------------------------------------
;F2 key (backup)
;------------------------------------------------------------------------------
fman_main_60:	cpi	tempreg1,0xf2		;F2
		brne	fman_main_70
		call	fsys_backup		
		rjmp	fman_main

;------------------------------------------------------------------------------
;F3 key (restore)
;------------------------------------------------------------------------------
fman_main_70:	cpi	tempreg1,0xf3		;F3
		brne	fman_main_80
		call	fsys_restore
		rjmp	fman_main

;------------------------------------------------------------------------------
;F4 key (rescan)
;------------------------------------------------------------------------------
fman_main_80:	cpi	tempreg1,0xf4		;F4
		brne	fman_main_90
		rjmp	fman_main
fman_main_90:	rjmp	fman_main_09

	
;------------------------------------------------------------------------------
;wait for key release
;------------------------------------------------------------------------------
fman_wnk:	lds	XL,libmio_keycode
		cpi	XL,0
		brne	fman_wnk		;wait for key release
		ret

;------------------------------------------------------------------------------
;standard menu bar 
;------------------------------------------------------------------------------
fman_smenu:	ldi	XL,0x27			;white on red
		libmio_color
		libmio_thistext
		.db 22,0," Exit ",0,0		;show message
		ldi	XL,0x60			;black on yellow
		libmio_color
		libmio_thistext
		.db 22,6,"Info ",0		;show message
		ldi	XL,0x40			;black on green
		libmio_color
		libmio_thistext
		.db 22,11,"Backup",0,0		;show message
		ldi	XL,0x17			;white on blue
		libmio_color
		libmio_thistext
		.db 22,17,"Restore",0		;show message
		ldi	XL,0x37			;white on blue
		libmio_color
		libmio_thistext
		.db 22,24,"Resca",0		;show message
		ldi	XL,0x6e			;white on magenta
		sts	libmio_vram+690+689,XL
		ldi	XL,'n'			;char
		sts	libmio_vram+689,XL
		ret	
		
;------------------------------------------------------------------------------
;control menu bar 
;------------------------------------------------------------------------------
fman_cmenu:	ldi	XL,0x27			;white on red
		libmio_color
		libmio_thistext
		.db 22,0,"      ",0,0		;show message
		ldi	XL,0x60			;black on yellow
		libmio_color
		libmio_thistext
		.db 22,6,"Delete",0,0		;show message
		ldi	XL,0x40			;black on green
		libmio_color
		libmio_thistext
		.db 22,12,"      ",0,0		;show message
		ldi	XL,0x17			;white on blue
		libmio_color
		libmio_thistext
		.db 22,18,"      ",0,0		;show message
		ldi	XL,0x37			;white on blue
		libmio_color
		libmio_thistext
		.db 22,24,"Forma",0		;show message
		ldi	XL,0x6e			;white on magenta
		sts	libmio_vram+690+689,XL
		ldi	XL,'t'			;char
		sts	libmio_vram+689,XL
		ret		

;------------------------------------------------------------------------------
;not formatted menu bar 
;------------------------------------------------------------------------------
fman_nfmenu:	ldi	XL,0x27			;white on red
		libmio_color
		libmio_thistext
		.db 22,0," Exit ",0,0		;show message
		ldi	XL,0x60			;black on yellow
		libmio_color
		libmio_thistext
		.db 22,6,"Format",0,0		;show message
		ldi	XL,0x40			;black on green
		libmio_color
		libmio_thistext
		.db 22,12,"      ",0,0		;show message
		ldi	XL,0x17			;white on blue
		libmio_color
		libmio_thistext
		.db 22,18,"      ",0,0		;show message
		ldi	XL,0x37			;white on blue
		libmio_color
		libmio_thistext
		.db 22,24,"Resca",0		;show message
		ldi	XL,0x6e			;white on magenta
		sts	libmio_vram+690+689,XL
		ldi	XL,'n'			;char
		sts	libmio_vram+689,XL
		ret		

;------------------------------------------------------------------------------
;no dataflash detected menu bar 
;------------------------------------------------------------------------------
fman_nomenu:	ldi	XL,0x27			;white on red
		libmio_color
		libmio_thistext
		.db 22,0," Exit ",0,0		;show message
		ldi	XL,0x60			;black on yellow
		libmio_color
		libmio_thistext
		.db 22,6,"      ",0,0		;show message
		ldi	XL,0x40			;black on green
		libmio_color
		libmio_thistext
		.db 22,12,"      ",0,0		;show message
		ldi	XL,0x17			;white on blue
		libmio_color
		libmio_thistext
		.db 22,18,"      ",0,0		;show message
		ldi	XL,0x37			;white on blue
		libmio_color
		libmio_thistext
		.db 22,24,"Resca",0		;show message
		ldi	XL,0x6e			;white on magenta
		sts	libmio_vram+690+689,XL
		ldi	XL,'n'			;char
		sts	libmio_vram+689,XL
		ret		
		
;------------------------------------------------------------------------------
;format medium 
;------------------------------------------------------------------------------
fman_fmt:	libmio_askthis
		.db	0x4f,"Format Medium?",0	;show message
		cpi	tempreg1,0x01
		breq	fman_fmt_1
		ret	
	
fman_fmt_1:	ldi	XL,0x4c			;color scheme (yellow on red)
		sts	libmio_color,XL
		ldi	YL,0x02
		ldi	YH,0x08
		ldi	ZL,0x1b
		ldi	ZH,0x0c
		libmio_wbox		
		libmio_thistext
		.db 9,3,"Formatting...      ",0	;show message
		libmio_thistext
		.db 11,3,0,0			;sst cursor
		call	fsys_format
		cpi	ereg,0x00		;error?
		brne	fman_fmt_err
fman_fmt_2:	libmio_thistext
		.db 9,3,"Formatting... Done!",0	;show message
fman_fmt_3:	ldi	XL,0xf4			;color scheme (red on white)
		sts	libmio_color,XL
		libmio_thistext
		.db 11,3,"   Press any key!  "0,0	;sst cursor
		libmio_waitkey
		ret

fman_fmt_err:	libmio_thistext
		.db 9,3,"Formatting... Error!",0,0	;show message
		rjmp	fman_fmt_3

;------------------------------------------------------------------------------
;medium info 
;------------------------------------------------------------------------------
fman_info:	ldi	XL,0xc0			;color scheme (black on yellow)
		sts	libmio_color,XL
		ldi	YL,4			;alert box
		ldi	YH,8
		ldi	ZL,25
		ldi	ZH,13
		libmio_wbox
		libmio_thistext
		.db 9,5,"Available:",0,0	;show message
		libmio_thistext
		.db 10,5,"Files: ",0		;show message
		ldi	ctrl,0x0c		;set format
		call	fsys_free
		push	YH			;save free pages
		push	YL
		movw	XL,ZL			;copy free files
		libmio_outdez
		libmio_thistext
		.db 255," /",0
		call	fsys_maxpage
		mov	XL,tempreg1
		ldi	XH,0x00
		adiw	XL,1
		libmio_outdez

fman_info_2:	libmio_thistext
		.db 11,5,"Blocks:",0		;show message
		pop	XL			;copy free data pages
		pop	XH
		libmio_outdez
		libmio_thistext
		.db 255," /",0		
		call	fsys_size
		ldi	XL,0xf0			;240
		mul	XL,tempreg1
		movw	XL,r0			;size
		sbiw	XL,0x01			;-1
		cpi	XH,0x00
		brne	fman_info_2a
		sbiw	XL,0x03
fman_info_2a:	libmio_outdez
fman_info_4:	ldi	XL,0x0c			;color scheme (inv. black on yellow)
		sts	libmio_color,XL
		libmio_thistext
		.db 12,5,"   Press any key!  "0,0	;sst cursor
		libmio_waitkey
		ret

		
	