;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# menu functions								#
;# copyright (c) 2006/2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.equ	menu_max = 6
;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
menu_main:	ldi	XL,1
		libmio_chpump			;enable chargepump for serial
		clr	ctrl			;normal mode
		sts	libmio_channel,byte0	;output to screen
		sts	libmio_vidmode,byte0	;standard video mode
		ldi	XL,0x2a			;color scheme (cyan on black)
		sts	libmio_color,XL
		libmio_clrscr
		ldi	YL,0			;maximal sized box
		ldi	YH,0
		ldi	ZL,29
		ldi	ZH,20
		libmio_whbox
		ldi	XL,0xa2			;color scheme (cyan on black)
		sts	libmio_color,XL
		libmio_thistext
		.db 1,4,0,0
		rcall	menu_itext1		;version
		
		rcall	menu_mbar		;draw menu bar at bottom line
		lds	tempreg1,libmio_lastmenu	
		andi	tempreg1,0x07		;only programs
		sts	libmio_lastmenu,tempreg1
		mov	tempreg3,tempreg1	;set as default
menu_main_1:	sbrc	tempreg3,7		;is below zero
		clr	tempreg3		;set to zero
		cpi	tempreg3,0x0b		;maximum+1
		brcs	menu_main_2
		ldi	tempreg3,0x0a		;limit to max
		
menu_main_2:	ldi	tempreg2,0x00		;prg icon counter
menu_main_3:	push	tempreg3
		rcall	menu_prgicon		;draw program icon
		pop	tempreg3
		inc	tempreg2
		cpi	tempreg2,0x08		;beyond limit?
		brne	menu_main_3
		rcall	menu_memicon		;dataflash
		rcall	menu_cfgicon		;config
		rcall	menu_inficon		;info	
		
menu_main_4:	push	tempreg3
		libmio_waitkey
		pop	tempreg3
		
		cpi	tempreg1,0xe2		;left
		brne	menu_main_5
		dec	tempreg3		;change to n-1
		rjmp	menu_main_1		;loop
				
menu_main_5:	cpi	tempreg1,0xe3		;right
		brne	menu_main_6
		inc	tempreg3		;change to n+1
		rjmp	menu_main_1		

menu_main_6:	cpi	tempreg1,0xe4		;up
		brne	menu_main_7
		cpi	tempreg3,0x04
		brcs	menu_main_7
		subi	tempreg3,0x04		;change to n-4
		rjmp	menu_main_1		
	    
menu_main_7:	cpi	tempreg1,0xe5		;down
		brne	menu_main_8
		cpi	tempreg3,0x07
		brcc	menu_main_8
		subi	tempreg3,0xfc		;change to n+4
		rjmp	menu_main_1		

menu_main_8:	cpi	tempreg3,0x08		;program
		brcs	menu_main_20		;branch if yes
		cpi	tempreg1,0xea		;enter
		breq	menu_main_9
		cpi	tempreg1,0xf4		;F4 (Run)
		brne	menu_main_4
				
menu_main_9:	cpi	tempreg3,0x08		;flash
		brne	menu_main_10
		rcall	menu_wnk
		call	fman_main		;call flash manager
		rcall	menu_wnk
		rjmp	menu_main								

menu_main_10:	cpi	tempreg3,0x09		;config
		brne	menu_main_11
		rcall	menu_wnk
		call	confp_main		;call config
		rcall	menu_wnk
		rjmp	menu_main								

menu_main_11:	rcall	menu_wnk
		call	infopage_main		;call info
		rcall	menu_wnk
		rjmp	menu_main								

		
;a program
menu_main_20:	cpi	tempreg1,0xea		;enter
		breq	menu_main_21
		cpi	tempreg1,0xf4		;run
		brne	menu_main_22
menu_main_21:	sts	libmio_lastmenu,tempreg3
		rcall	menu_wnk
		clr	ereg
		call	menu_runprog		;run pogram
		rcall	menu_wnk
		rjmp	menu_main

menu_main_22:	cpi	tempreg1,0xf1		;edit
		brne	menu_main_23
		sts	libmio_lastmenu,tempreg3
		rcall	menu_wnk
		clr	ereg
		call	menu_editloop		;edit program
		rcall	menu_wnk
		rjmp	menu_main

menu_main_23:	cpi	tempreg1,0xf2		;save
		brne	menu_main_24
		sts	libmio_lastmenu,tempreg3
		rcall	menu_wnk
		call	fsys_savebas
		rcall	menu_wnk
		rjmp	menu_main

menu_main_24:	cpi	tempreg1,0xf3		;load
		brne	menu_main_25
		sts	libmio_lastmenu,tempreg3
		rcall	menu_wnk
		call	fsys_loadbas		;edit program
		rcall	menu_wnk
		rjmp	menu_main
		
menu_main_25:	rjmp	menu_main_4												
				

;------------------------------------------------------------------------------
; editor loop
;------------------------------------------------------------------------------
menu_editloop:	lds	XL,libmio_keycode
		cpi	XL,0
		brne	menu_editloop
		call	editor_main
		sts	libmio_kflags,byte0
		lds	XL,libmio_lastkey
		cpi	XL,0xf4			;F4
		breq	menu_editloop1
		rjmp	menu_main
menu_editloop1:	call	menu_runprog		;run programm
menu_editloop2:	libmio_waitkey
		cpi	tempreg1,0xed		;ESC
		brne	menu_editloop2
		sts	libmio_vidmode,byte0	;set to text mode
		rjmp	menu_editloop

		
;------------------------------------------------------------------------------
;run program
;------------------------------------------------------------------------------
menu_runprog:	libmio_clrscr			;clear screen from menu			
		call	tbrun			;run programm
		sts	libmio_seqspeed,byte0	;stop sequencer
		libmio_nmode			;normal mode
		sts	libmio_channel,byte0	;set output to screen
		libmio_newline
		ldi	XL,0x27
		libmio_color
		ldi	ctrl,0x02		;normal height
		lds	XL,libmio_vidmode
		cpi	XL,0x00
		breq	menu_runprog2
		cpi	XL,0x03
		brne	menu_runprog1
		ldi	XL,0x4e
		sts	libmio_color,XL
		rjmp	menu_runprog2
		
menu_runprog1:	ldi	XL,0x20
		sts	libmio_color,XL
		ldi	XL,0x40			;red
		sts	0x100,XL
		ldi	XL,0xe0			;white
		sts	0x101,XL
			
menu_runprog2:	lds	XL,libmio_vidmode	;get video mode
		cpi	XL,0x00
		brne	menu_runprog3
		libmio_thistext
		.db 22,0," PRESS ESC ",0
		ret	

menu_runprog3:	cpi	XL,0x01
		brne	menu_runprog4
		libmio_thistext
		.db 105,0," PRESS ESC ",0
		ret	

menu_runprog4:	cpi	XL,0x02
		brne	menu_runprog5
		libmio_thistext
		.db 66,0," PRESS ESC ",0
		ret	

menu_runprog5:	libmio_thistext
		.db 47,0," PRESS ESC ",0
		ret	
		

;------------------------------------------------------------------------------
;info text (version)
;------------------------------------------------------------------------------
menu_itext1:	libmio_thistext
		.db 255,"AVR-ChipBasic2 V0.69",0
		ret

;------------------------------------------------------------------------------
;show first bytes of first program (debug)
;------------------------------------------------------------------------------
		libmio_thistext
		.db 20,1,0,0
		ldi	r16,10	
		ldi	ZL,low(0x9020)
		ldi	ZH,high(0x9020)
mxv1:		clr	ctrl
		lpm	XL,Z+
		clr	XH
		libmio_outhex
		libmio_outspace
		dec	r16
		brne	mxv1				
		ret

;------------------------------------------------------------------------------
;wait for key release
;------------------------------------------------------------------------------
menu_wnk:	lds	XL,libmio_keycode
		cpi	XL,0
		brne	menu_wnk		;wait for key release
		ret

;------------------------------------------------------------------------------
;draw program icon (tempreg2=number, tempreg3=invert 0/1)
;------------------------------------------------------------------------------
menu_prgicon:	ldi	XL,0x01			;X coord of icon 0
		ldi	XH,0x03			;Y coord of icon 0
		sbrc	tempreg2,0
		subi	XL,0xf9			;X+7
		sbrc	tempreg2,1
		subi	XL,0xf2			;X+14
		sbrc	tempreg2,2
		subi	XH,0xfa			;Y+6
		ldi	tempreg1,0x2e		;white on blue
		cp	tempreg3,tempreg2
		brne	menu_prgicon_0
		swap	tempreg1
menu_prgicon_0:	sts	libmio_color,tempreg1
		libmio_gotoxy
		libmio_thistext
		.db	255,0x20,0x20,0
		ldi	tempreg1,'P'
		libmio_outchar
		mov	tempreg1,tempreg2	;prg number
		subi	tempreg1,0xcf		;+0x31
		libmio_outchar
		libmio_thistext
		.db	255,0x20,0x20,0
		inc	XH
		libmio_gotoxy
		libmio_thistext
		.db	255,0x20,0x20,0xd0,0xd1,0x20,0x20,0
		inc	XH
		libmio_gotoxy
		libmio_thistext
		.db	255,0x20,0x20,0xd2,0xd3,0x20,0x20,0
		ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		sbrc	tempreg2,0
		subi	ZH,0xf4			;+3072
		sbrc	tempreg2,1
		subi	ZH,0xe8			;+6144
		sbrc	tempreg2,2
		subi	ZH,0xd0			;+12288
		inc	XH
		libmio_gotoxy
		ldi	tempreg3,0x06		;chars per line
menu_prgicon_1:	lpm	tempreg1,Z+
		libmio_outchar	
		dec	tempreg3
		brne	menu_prgicon_1
		inc	XH
		libmio_gotoxy
		ldi	tempreg3,0x06		;chars per line
menu_prgicon_2:	lpm	tempreg1,Z+
		libmio_outchar	
		dec	tempreg3
		brne	menu_prgicon_2
		inc	XH
		libmio_gotoxy
		libmio_thistext
		.db	255,0x20,0x20,0x20,0x20,0x20,0x20,0
		ret					
		
;------------------------------------------------------------------------------
;draw info icon (X=coord)
;------------------------------------------------------------------------------
menu_inficon:	ldi	tempreg1,0x2a		;cyan on blue
		cpi	tempreg3,0x0a
		brne	menu_inficon_1
		swap	tempreg1
menu_inficon_1:	sts	libmio_color,tempreg1
		libmio_thistext
		.db	15,15,0x20,0x20,0x20,0x20,0x20,0x20,0,0
		libmio_thistext
		.db	16,15,0x20,0x20,0xd4,0xd5,0x20,0x20,0,0
		libmio_thistext
		.db	17,15,0x20,0x20,0xd6,0xd7,0x20,0x20,0,0
		libmio_thistext
		.db	18,15," INFO ",0,0	
		libmio_thistext
		.db	19,15,0x20,0x20,0x20,0x20,0x20,0x20,0,0
		ret		


;------------------------------------------------------------------------------
;draw conf icon (X=coord)
;------------------------------------------------------------------------------
menu_cfgicon:	ldi	tempreg1,0x28		;green on blue
		cpi	tempreg3,0x09
		brne	menu_cfgicon_1
		swap	tempreg1
menu_cfgicon_1:	sts	libmio_color,tempreg1
		libmio_thistext
		.db	15,8,0x20,0x20,0x20,0x20,0x20,0x20,0,0
		libmio_thistext
		.db	16,8,0x20,0x20,0xd8,0xd9,0x20,0x20,0,0
		libmio_thistext
		.db	17,8,0x20,0x20,0xda,0xdb,0x20,0x20,0,0
		libmio_thistext
		.db	18,8,"CONFIG",0,0		
		libmio_thistext
		.db	19,8,0x20,0x20,0x20,0x20,0x20,0x20,0,0
		ret		


;------------------------------------------------------------------------------
;draw flash icon (X=coord)
;------------------------------------------------------------------------------
menu_memicon:	ldi	tempreg1,0x2c		;yellow on blue
		cpi	tempreg3,0x08
		brne	menu_memicon_1
		swap	tempreg1
menu_memicon_1:	sts	libmio_color,tempreg1
		libmio_thistext
		.db	15,1,0x20,0x20,0x20,0x20,0x20,0x20,0,0
		libmio_thistext
		.db	16,1,0x20,0x20,0xdc,0xdd,0x20,0x20,0,0
		libmio_thistext
		.db	17,1,0x20,0x20,0xde,0xdf,0x20,0x20,0,0
		libmio_thistext
		.db	18,1,"DFLASH",0,0
		libmio_thistext
		.db	19,1,0x20,0x20,0x20,0x20,0x20,0x20,0,0
		ret		
									

;------------------------------------------------------------------------------
;the menu bar at bottom line
;------------------------------------------------------------------------------
menu_mbar:	ldi	XL,0x00			;black
		libmio_color
		libmio_thistext
		.db 21,0,"                              ",0,0	;black space

		ldi	XL,0x27			;white on red
		libmio_color
		libmio_thistext
		.db 22,0,"      ",0,0		;show message
		ldi	XL,0x60			;black on yellow
		libmio_color
		libmio_thistext
		.db 22,6," Edit ",0,0		;show message
		ldi	XL,0x40			;black on green
		libmio_color
		libmio_thistext
		.db 22,12," Save ",0,0		;show message
		ldi	XL,0x17			;white on blue
		libmio_color
		libmio_thistext
		.db 22,18," Load ",0,0		;show message
		ldi	XL,0x6e			;white on magenta
		sts	libmio_vram+690+689,XL
		ldi	XL,0x37			;white on magenta
		libmio_color
		libmio_thistext
		.db 22,24," Run ",0		;show message
		ret
			